import ftplib
import os
from copy import deepcopy

from shipyard_templates import CloudStorage, ShipyardLogger

from shipyard_ftp import exceptions

logger = ShipyardLogger().get_logger()


class FtpClient(CloudStorage):
    def __init__(self, host: str, user: str, pwd: str, port: int = None) -> None:
        self._client = None
        self.host = host
        self.user = user
        self.pwd = pwd
        self.port = port or 21

    @property
    def client(self):
        """
        Returns the current FTP client if it exists, otherwise creates a new one

        :return: ftplib.FTP
        """
        if self._client is None:
            self._client = self.get_client()
        return self._client

    def connect(self):
        try:
            self.get_client()
        except Exception:
            return 1
        else:
            return 0

    def move(self, source_full_path, destination_full_path):
        """
        Move a single file from one directory of the ftp server to another
        """
        current_dir = self.client.pwd()
        source_path = os.path.normpath(os.path.join(current_dir, source_full_path))
        dest_path = os.path.normpath(os.path.join(current_dir, destination_full_path))
        try:
            self.client.rename(source_path, dest_path)
        except Exception as e:
            raise exceptions.MoveError(
                f"Failed to move {source_path}. Ensure that the source/destination file name and folder name are "
                f"correct. Error message from server: {e} ",
            ) from e

        logger.info(f"{source_path} successfully moved to " f"{dest_path}")

    def remove(self, file_path):
        try:
            self.client.delete(file_path)
            logger.info(f"Successfully deleted {file_path}")
        except Exception as e:
            raise exceptions.DeleteError(
                f"Failed to delete file {file_path}. Ensure that the folder path and file name our correct"
                f"Message from server: {e}"
            )

    def upload(self, source_full_path, destination_full_path):
        try:
            with open(source_full_path, "rb") as f:
                self.client.storbinary(f"STOR {destination_full_path}", f)
        except Exception as e:
            raise exceptions.UploadError(f"Failed to upload {source_full_path} to FTP server") from e

        logger.info(f"{source_full_path} successfully uploaded to " f"{destination_full_path}")

    def get_client(self):
        """
        Attempts to create an FTP client at the specified host with the
        specified credentials
        """
        logger.info(f"Connecting to FTP server at {self.host}:{self.port} with user {self.user}...")
        try:
            client = ftplib.FTP(timeout=300)
            client.connect(self.host, int(self.port))
            client.login(self.user, self.pwd)
            client.set_pasv(True)
            client.set_debuglevel(0)
            logger.info(f"Connected to FTP server at {self.host}:{self.port} successfully.")
            return client
        except Exception as e:
            raise exceptions.InvalidCredentials(
                f"Error accessing the FTP server with the specified credentials\n"
                f"The server says: {e}")from e

    def download(self, file_name, destination_full_path=None):
        """
        Download a selected file from the FTP server to local storage in
        the current working directory or specified path.
        """
        if not self.client:
            self.get_client()

        os.makedirs(os.path.dirname(destination_full_path), exist_ok=True)
        try:
            logger.info(f"Attempting to download {file_name}...")
            with open(destination_full_path, "wb") as f:
                self.client.retrbinary(f"RETR {file_name}", f.write)
            logger.info(f"{file_name} successfully downloaded to {destination_full_path}")
        except Exception as e:
            os.remove(destination_full_path)
            raise exceptions.DownloadError(f"Failed to download {file_name}. Message from server: {e}") from e

        logger.info(f"{file_name} successfully downloaded to {destination_full_path}")

    def create_new_folders(self, destination_path):
        """
        Changes working directory to the specified destination path
        and creates it if it doesn't exist
        """
        if not self.client:
            self.get_client()

        original_dir = self.client.pwd()
        for folder in destination_path.split("/"):
            try:
                self.client.cwd(folder)
            except Exception:
                self.client.mkd(folder)
                self.client.cwd(folder)
        self.client.cwd(original_dir)

    def find_files_in_directory(self, client, folder_filter, files, folders):
        """
        Pull in a list of all entities under a specific directory and categorize them into files and folders.

        Does this by changing the directory to the entity name. If it can't, it's a file.
        """
        original_dir = self.client.pwd()
        names = self.client.nlst(folder_filter)
        for name in names:
            if all(char == "." for char in name):  # Skip relative directories
                continue

            if "/" not in name:
                # Accounts for an issue where some FTP servers return file names
                # without folder prefixes.
                name = f"{folder_filter}/{name}"

            try:
                # If you can change the directory to the entity_name, it's a
                # folder.
                client.cwd(name)
                folders.append(name)
            except ftplib.error_perm:
                files.append(name)
                continue
            client.cwd(original_dir)

        folders.remove(folder_filter)

        return files, folders

    def is_file(self, filename):
        """
        Helper function to check if the path in a ftp server is a file or a directory.
        If it is a directory, ftp.size returns None
        """
        try:
            if self.client.size(filename) is not None:
                return True
        except Exception:
            return False

    def get_all_nested_items(self, working_directory, dir_list=None):
        """
        Recursive function to get all the nested files and directories on the FTP server.
        @params:
            client: FTP client
            working_directory: the folder path to start the function call
        """
        if dir_list is None:
            dir_list = []
        dirs = deepcopy(dir_list)
        paths = self.client.nlst(working_directory)
        for path in paths:
            if not self.is_file(path):
                dirs = self.get_all_nested_items(path, dir_list=dirs)
            else:
                dirs.append(path)
        return dirs


if __name__ == "__main__":
    from dotenv import load_dotenv, find_dotenv

    load_dotenv(find_dotenv())
    credentials = {"host": os.getenv("FTP_HOST"), "port": os.getenv("FTP_PORT"), "user": os.getenv("FTP_USERNAME"),
                   "pwd": os.getenv("FTP_PASSWORD"), }

    ftp = FtpClient(credentials["host"], credentials["user"], credentials["pwd"], credentials["port"])

    print(ftp.connect())



from shipyard_templates import ShipyardLogger, ExitCodeException, CloudStorage

logger = ShipyardLogger().get_logger()
EXIT_CODE_FTP_MOVE_ERROR = 100
EXIT_CODE_FTP_DELETE_ERROR = 101
EXIT_CODE_UPLOAD_ERROR = CloudStorage.EXIT_CODE_UPLOAD_ERROR
EXIT_CODE_NO_MATCHES_FOUND = CloudStorage.EXIT_CODE_FILE_MATCH_ERROR
EXIT_CODE_INCORRECT_CREDENTIALS = CloudStorage.EXIT_CODE_INVALID_CREDENTIALS
EXIT_CODE_DOWNLOAD_ERROR= CloudStorage.EXIT_CODE_DOWNLOAD_ERROR


class MoveError(ExitCodeException):
    def __init__(self, message):
        super().__init__(message, EXIT_CODE_FTP_MOVE_ERROR)


class UploadError(ExitCodeException):
    def __init__(self, message):
        super().__init__(message, EXIT_CODE_UPLOAD_ERROR)


class NoMatchFound(ExitCodeException):
    def __init__(self, message):
        super().__init__(message, EXIT_CODE_NO_MATCHES_FOUND)

class InvalidCredentials(ExitCodeException):
    def __init__(self, message):
        super().__init__(message, EXIT_CODE_INCORRECT_CREDENTIALS)

class DownloadError(ExitCodeException):
    def __init__(self, message):
        super().__init__(message, EXIT_CODE_DOWNLOAD_ERROR)


class DeleteError(ExitCodeException):
    def __init__(self, message):
        super().__init__(message, EXIT_CODE_FTP_DELETE_ERROR)


import argparse
import re
import sys

from shipyard_bp_utils import files as shipyard
from shipyard_templates import ExitCodeException, CloudStorage
from shipyard_templates.shipyard_logger import ShipyardLogger

from shipyard_ftp.ftp import FtpClient

logger = ShipyardLogger().get_logger()

EXIT_CODE_INCORRECT_CREDENTIALS = 3
EXIT_CODE_NO_MATCHES_FOUND = 200


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        choices={"exact_match", "regex_match"},
        required=True,
    )
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default=None,
        required=False,
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument("--host", dest="host", default=None, required=True)
    parser.add_argument("--port", dest="port", default=21, required=True)
    parser.add_argument("--username", dest="username", default=None, required=False)
    parser.add_argument("--password", dest="password", default=None, required=False)
    return parser.parse_args()


def find_matching_files(file_names, file_name_re):
    """
    Return a list of all file_names that matched the regular expression.
    """
    matching_file_names = []
    for file_name in file_names:
        fname = file_name.rsplit("/", 1)[-1]
        if re.search(file_name_re, fname):
            matching_file_names.append(file_name)

    return matching_file_names


def main():
    try:
        args = get_args()
        host = args.host
        port = args.port
        username = args.username
        password = args.password
        source_file_name = args.source_file_name
        source_folder_name = shipyard.clean_folder_name(args.source_folder_name)
        source_full_path = shipyard.combine_folder_and_file_name(
            folder_name=source_folder_name, file_name=source_file_name
        )
        source_file_name_match_type = args.source_file_name_match_type

        destination_folder_name = shipyard.clean_folder_name(args.destination_folder_name)
        if destination_folder_name:
            shipyard.create_folder_if_dne(destination_folder_name)
        client = FtpClient(host=host, port=port, user=username, pwd=password)

        logger.info("Beginning the download process...")
        if source_file_name_match_type == "exact_match":
            try:
                destination_name = shipyard.determine_destination_full_path(
                    destination_folder_name=destination_folder_name,
                    destination_file_name=args.destination_file_name,
                    source_full_path=source_full_path)
                client.download(source_full_path, destination_name)

            except Exception as e:
                logger.error(f"Failed to download {source_full_path} due to {e}\n"
                             f"Most likely, the file name/folder name you specified has typos or the full folder name was "
                             f"not provided. Check these and try again.")
                raise e
        elif source_file_name_match_type == "regex_match":
            folders = [source_folder_name]
            files = []
            while folders:
                folder_filter = folders[0]
                files, folders = client.find_files_in_directory(
                    folder_filter=folder_filter, files=files, folders=folders
                )

            matching_file_names = shipyard.find_all_file_matches(files, re.compile(source_file_name))

            number_of_matches = len(matching_file_names)

            if number_of_matches == 0:
                logger.info(f'No matches were found for regex "{source_file_name}".')
                sys.exit(EXIT_CODE_NO_MATCHES_FOUND)

            logger.info(f"{len(matching_file_names)} files found. Preparing to download...")

            for index, file_name in enumerate(matching_file_names):
                destination_name = shipyard.determine_destination_full_path(
                    destination_folder_name=destination_folder_name,
                    destination_file_name=args.destination_file_name,
                    source_full_path=file_name,
                    file_number=index + 1,
                )

                logger.info(f"Downloading file {index + 1} of {len(matching_file_names)}")

                client.download(file_name, destination_name)

        logger.info("Download process complete.")
    except ExitCodeException as e:
        logger.error(e.message)
        sys.exit(e.exit_code)
    except Exception as e:
        logger.error(f"An error occurred during the download process: {e}")
        sys.exit(CloudStorage.EXIT_CODE_UNKNOWN_ERROR)


if __name__ == "__main__":
    main()



import argparse
import re
import sys

from shipyard_bp_utils import files as shipyard
from shipyard_templates import ShipyardLogger, ExitCodeException, CloudStorage

from shipyard_ftp.exceptions import EXIT_CODE_NO_MATCHES_FOUND
from shipyard_ftp.ftp import FtpClient

logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()

    parser.add_argument("--host", dest="host", default=None, required=True)
    parser.add_argument("--port", dest="port", default=21, required=True)
    parser.add_argument("--username", dest="username", default=None, required=False)
    parser.add_argument("--password", dest="password", default=None, required=False)
    parser.add_argument(
        "--file-name-match-type",
        dest="file_name_match_type",
        choices={"exact_match", "regex_match"},
        required=False,
        default="exact_match",
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=True
    )
    return parser.parse_args()


def main():
    try:
        args = get_args()
        host = args.host
        port = args.port
        username = args.username
        password = args.password
        file_name_match_type = args.file_name_match_type
        file_name = args.source_file_name
        folder_name = shipyard.clean_folder_name(args.source_folder_name)

        client = FtpClient(host=host, port=port, user=username, pwd=password)

        if file_name_match_type == "regex_match":
            folders = [folder_name]
            files = []
            while folders:
                folder_filter = folders[0]

                files, folders = client.find_files_in_directory(
                    folder_filter=folder_filter, files=files, folders=folders)

            matching_file_names = shipyard.find_all_file_matches(
                files, re.compile(file_name)
            )

            number_of_matches = len(matching_file_names)

            if number_of_matches == 0:
                logger.info(f'No matches were found for regex "{file_name}".')
                sys.exit(EXIT_CODE_NO_MATCHES_FOUND)

            for index, file_name in enumerate(matching_file_names):
                logger.info(
                    f"Deleting file {index + 1} of {len(matching_file_names)} out of {number_of_matches}"
                )
                try:
                    client.remove(file_name)
                except Exception:
                    logger.error(f"Failed to delete {file_name}... Skipping")
        elif file_name_match_type == "exact_match":
            file_path = shipyard.combine_folder_and_file_name(folder_name, file_name)
            try:
                client.remove(file_path)
            except Exception as e:
                logger.error(
                    "Most likely, the file name/folder name you specified has typos or the full folder name was not "
                    "provided. Check these and try again."
                )
                raise e
    except ExitCodeException as e:
        logger.error(e.message)
        sys.exit(e.exit_code)
    except Exception as e:
        logger.error(f"An unexpected error occurred: {e}")
        sys.exit(CloudStorage.EXIT_CODE_UNKNOWN_ERROR)


if __name__ == "__main__":
    main()



import argparse
import os
import re
import sys

from shipyard_bp_utils import files as shipyard
from shipyard_templates import ShipyardLogger, ExitCodeException, CloudStorage

from shipyard_ftp.ftp import FtpClient

logger = ShipyardLogger().get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        choices={"exact_match", "regex_match"},
        required=True,
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default=None,
        required=False,
    )
    parser.add_argument("--host", dest="host", default=None, required=True)
    parser.add_argument("--port", dest="port", default=21, required=True)
    parser.add_argument("--username", dest="username", default=None, required=False)
    parser.add_argument("--password", dest="password", default=None, required=False)
    return parser.parse_args()


def main():
    try:
        args = get_args()
        host = args.host
        port = args.port
        username = args.username
        password = args.password
        source_file_name = args.source_file_name
        source_folder_name = args.source_folder_name
        destination_filename = args.destination_file_name or source_file_name
        destination_folder_name = shipyard.clean_folder_name(args.destination_folder_name)
        source_file_name_match_type = args.source_file_name_match_type
        ftp_client = FtpClient(host=host, port=port, user=username, pwd=password)
        if source_file_name_match_type == "exact_match":
            source_full_path = shipyard.combine_folder_and_file_name(
                folder_name=f"{os.getcwd()}/{source_folder_name}", file_name=source_file_name
            )
            destination_full_path = shipyard.determine_destination_full_path(
                destination_folder_name=destination_folder_name,
                destination_file_name=destination_filename,
                source_full_path=source_full_path,
            )
            if len(destination_full_path.split("/")) > 1:
                path, file_name = destination_full_path.rsplit("/", 1)
                ftp_client.create_new_folders(path)

            ftp_client.upload(source_full_path, destination_full_path)

        elif source_file_name_match_type == "regex_match":
            file_names = shipyard.find_all_local_file_names(source_folder_name)
            matching_file_names = shipyard.find_all_file_matches(
                file_names, re.compile(source_file_name)
            )

            number_of_matches = len(matching_file_names)

            if number_of_matches == 0:
                logger.error(f'No matches were found for regex "{source_file_name}".')
                sys.exit(ftp_client.EXIT_CODE_FILE_MATCH_ERROR)

            logger.info(f"{len(matching_file_names)} files found. Preparing to upload...")

            for index, key_name in enumerate(matching_file_names):
                destination_full_path = shipyard.determine_destination_full_path(
                    destination_folder_name=destination_folder_name,
                    destination_file_name=destination_filename,
                    source_full_path=key_name,
                    file_number=index + 1,
                )
                if len(destination_full_path.split("/")) > 1:
                    path, file_name = destination_full_path.rsplit("/", 1)
                    ftp_client.create_new_folders(destination_path=path)
                logger.info(f"Uploading file {index + 1} of {len(matching_file_names)}")
                ftp_client.upload(key_name, destination_full_path)
    except ExitCodeException as e:
        logger.error(e.message)
        sys.exit(e.exit_code)
    except Exception as e:
        logger.error(f"An error occurred: {e}")
        sys.exit(CloudStorage.EXIT_CODE_UNKNOWN_ERROR)


if __name__ == "__main__":
    main()



import os
import sys
from shipyard_ftp import FtpClient


def get_args():
    return {
        "host": os.getenv("FTP_HOST"),
        "port": os.getenv("FTP_PORT"),
        "username": os.getenv("FTP_USERNAME"),
        "password": os.getenv("FTP_PASSWORD"),
    }


def main():
    args = get_args()
    host = args["host"]
    port = args["port"]
    user = args["username"]
    pwd = args["password"]
    ftp = FtpClient(host, user, pwd, port)
    try:
        ftp.connect()
        ftp.logger.info("Successfully connected to FTP server")
        sys.exit(0)
    except Exception as e:
        ftp.logger.error("Could not connect to FTP server with the given credentials")
        sys.exit(1)


if __name__ == "__main__":
    main()



import argparse
import re
import sys
from copy import deepcopy

from shipyard_bp_utils import files as shipyard
from shipyard_templates import ShipyardLogger

from shipyard_ftp.ftp import FtpClient

logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        choices={"exact_match", "regex_match"},
        default="exact_match",
        required=False,
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default=None,
        required=False,
    )
    parser.add_argument("--host", dest="host", default=None, required=True)
    parser.add_argument("--port", dest="port", default=21, required=True)
    parser.add_argument("--username", dest="username", default=None, required=False)
    parser.add_argument("--password", dest="password", default=None, required=False)
    return parser.parse_args()


def get_all_nested_items(client, working_directory, dir_list=None):
    """
    Recursive function to get all the nested files and directories on the FTP server.
    @params:
        client: FTP client
        working_directory: the folder path to start the function call
    """
    if dir_list is None:
        dir_list = []
    dirs = deepcopy(dir_list)
    paths = client.nlst(working_directory)
    for path in paths:
        if not client.is_file(path):
            dirs = get_all_nested_items(client, path, dir_list=dirs)
        else:
            dirs.append(path)
    return dirs


def main():
    args = get_args()
    host = args.host
    port = args.port
    username = args.username
    password = args.password
    source_file_name = args.source_file_name
    source_folder_name = args.source_folder_name

    destination_folder_name = shipyard.clean_folder_name(
        args.destination_folder_name
    )
    source_file_name_match_type = args.source_file_name_match_type
    ftp_client = FtpClient(host=host, port=port, user=username, pwd=password)
    # if the source folder name is omitted, then use the current working directory
    source_folder_name = source_folder_name or ftp_client.client.pwd()
    if source_file_name_match_type == "exact_match":
        source_full_path = shipyard.combine_folder_and_file_name(
            source_folder_name, source_file_name
        )
        destination_file = shipyard.determine_destination_file_name(
            source_full_path=source_full_path,
            destination_file_name=args.destination_file_name,
        )
        destination_full_path = shipyard.combine_folder_and_file_name(
            destination_folder_name,
            destination_file,
        )
        if len(destination_full_path.split("/")) > 1:
            path, file_name = destination_full_path.rsplit("/", 1)
            ftp_client.create_new_folders(destination_path=path)
        ftp_client.move(
            source_full_path=source_full_path,
            destination_full_path=destination_full_path,
        )


    elif source_file_name_match_type == "regex_match":
        file_names = get_all_nested_items(ftp_client.client, source_folder_name)
        matching_file_names = shipyard.find_all_file_matches(
            file_names, re.compile(source_file_name)
        )

        number_of_matches = len(matching_file_names)

        if number_of_matches == 0:
            logger.info(f'No matches were found for regex "{source_file_name}".')
            sys.exit(ftp_client.EXIT_CODE_FILE_MATCH_ERROR)

        logger.info(f"{len(matching_file_names)} files found. Preparing to move...")

        for index, key_name in enumerate(matching_file_names, 1):
            destination_full_path = shipyard.determine_destination_full_path(
                destination_folder_name=destination_folder_name,
                destination_file_name=args.destination_file_name,
                source_full_path=key_name,
                file_number=None if len(matching_file_names) == 1 else index,
            )
            if len(destination_full_path.split("/")) > 1:
                path, file_name = destination_full_path.rsplit("/", 1)
                ftp_client.create_new_folders(path)
            logger.info(f"Moving file {index} of {len(matching_file_names)}")
            ftp_client.move(
                source_full_path=key_name, destination_full_path=destination_full_path
            )


if __name__ == "__main__":
    main()
